import * as React from "react";
import {ViroARPlane, ViroNode, ViroARPlaneSizes, ViroPlaneUpdatedMap, ViroQuad} from "@viro-community/react-viro";
import {ViroCommonProps, ViroObjectProps} from "@viro-community/react-viro/dist/components/AR/ViroCommonProps";
import {Viro3DPoint, ViroARPlaneType, ViroNativeRef} from "@viro-community/react-viro/dist/components/Types/ViroUtils";
import {ViroMaterials} from "@viro-community/react-viro/components/Material/ViroMaterials";

const _maxPlanes = 15;
const _planePrefix = "ViroARPlaneSelector_Plane_";

type Props = ViroCommonProps &
    ViroObjectProps & {
  maxPlanes?: number;
  minHeight?: number;
  minWidth?: number;
  alignment?:
      | "Horizontal"
      | "HorizontalUpward"
      | "HorizontalDownward"
      | "Vertical";

  onPlaneSelected?: (updateMap: ViroPlaneUpdatedMap) => void;
  onObjectPlaced?: (position: Viro3DPoint) => void;
};

type State = {
  selectedPlane: number;
  pointOnPlane: Viro3DPoint;
  foundARPlanes: ViroARPlaneType[];
  arPlaneSizes: ViroARPlaneSizes;
};

export class ViroARObjectPlacer extends React.Component<Props, State> {
  component: ViroNativeRef = null;

  state: State = {
    selectedPlane: -1,
    pointOnPlane: null,
    foundARPlanes: [],
    arPlaneSizes: [],
  };

  render() {
    // Uncomment this line to check for misnamed props
    // checkMisnamedProps("ViroARPlaneSelector", this.props);

    return <ViroNode>{this.getARPlanes()}</ViroNode>;
  }

  getARPlanes() {
    if (this.state.selectedPlane === -1) {
      const arPlanes = [];
      const numPlanes = this.props.maxPlanes || _maxPlanes;
      for (let i = 0; i < numPlanes; i++) {
        const foundARPlane = this.state.foundARPlanes[i];
        const planeWidth = foundARPlane?.width ?? 0;
        const planeHeight = foundARPlane?.height ?? 0;
        const planePosition = foundARPlane?.center ?? [0, 0, 0];
        arPlanes.push(
            <ViroARPlane
                key={_planePrefix + i}
                minWidth={this.props.minWidth}
                minHeight={this.props.minHeight}
                alignment={this.props.alignment}
                onAnchorUpdated={this.onARPlaneUpdated(i)}
            >
              <ViroQuad
                  materials={"ArtdefaAR_Translucent"}
                  onClick={this.selectPlane(i)}
                  position={planePosition}
                  width={planeWidth}
                  height={planeHeight}
                  rotation={[-90, 0, 0]}
              />
            </ViroARPlane>
        );
      }
      return arPlanes;
    } else if (this.state.pointOnPlane === null) {

      const planeIdx = this.state.selectedPlane;

      const foundARPlane = this.state.foundARPlanes[planeIdx];
      const planeWidth = foundARPlane?.width ?? 0;
      const planeHeight = foundARPlane?.height ?? 0;
      const planePosition = foundARPlane?.center ?? [0, 0, 0];

      return (
          <ViroARPlane
              key={_planePrefix + planeIdx}
              onClick={this.selectPointOnPlane}
              minWidth={this.props.minWidth}
              minHeight={this.props.minHeight}
              alignment={this.props.alignment}
              onAnchorUpdated={this.onARPlaneUpdated(planeIdx)}
          >
            <ViroQuad
                materials={"ArtdefaAR_Translucent"}
                position={planePosition}
                width={planeWidth}
                height={planeHeight}
                rotation={[-90, 0, 0]}
            />
          </ViroARPlane>
      )
    } else return (
        <ViroARPlane
            key={_planePrefix + this.state.selectedPlane}
            {...this.props}
        ></ViroARPlane>
    );
  }

  selectPlane = (index: number) => {
    return () => {
      this.setState({ selectedPlane: index });
      this.onPlaneSelected(this.state.foundARPlanes[index]);
    };
  };

  selectPointOnPlane = (position: Viro3DPoint) => {
    const plane = this.state.foundARPlanes[this.state.selectedPlane];
    const relativePosition = position.map(function (item, index) {
      return item - plane.position[index];
    }) as Viro3DPoint;
    this.setState({pointOnPlane: relativePosition});
    this.onObjectPlaced(relativePosition);
  }

  onARPlaneUpdated = (index: number) => {
    return (updateMap: ViroPlaneUpdatedMap) => {
      this.setState({
        foundARPlanes: [
          ...this.state.foundARPlanes.slice(0, index),
          updateMap,
          ...this.state.foundARPlanes.slice(index),
        ],
        arPlaneSizes: this.state.arPlaneSizes,
      });
    };
  };

  onPlaneSelected = (updateMap: ViroPlaneUpdatedMap) => {
    this.props.onPlaneSelected && this.props.onPlaneSelected(updateMap);
  };

  onObjectPlaced = (position: Viro3DPoint) => {
    this.props.onObjectPlaced && this.props.onObjectPlaced(position);
  };

  /*
  This function allows the user to reset the plane and select a new plane.
  */
  reset = () => {
    this.setState({
      selectedPlane: -1,
    });
  };
}

ViroMaterials.createMaterials({
  ArtdefaAR_Translucent: {
    lightingModel: "Constant",
    diffuseColor: "#88888888",
  },
});
